function [Httau, GroupDelay]=RayleighSelective(Ns, Ts, Fd, Tsym, Tau, PdB);

% Function Httau=RayleighSelective(Ns,Ts, Fd, Tsym, Tau, PdB);
% Generate 2-D complex channel impulse response for frequency selective
% Rayleigh fading channels. Total power of the fading reponse is normalized to 0 dB. 
% Pulse shaping filter = raised cosine filter with rolloff=0.35

% Inputs:
%    Ns : number of samples in t-domain for the generated fading impulse response
%    Ts : Sampling Interval (in seconds)
%    Fd : Maxmimum Doppler Frequency of each ray (in Hz)
%   Tsym: Symbol interval in seconds
%   Tau:  multipath delay vector (seconds)
%   PdB : Multipath power vector (in dB)

% Output:
%   Httau : complex channel impulse response matrix with size  
%   (max(Tau)/Tsym+FilterLength)*N_up by Ns 
%   GroupDelay: filter delay in tau domain

% Created by Y. R. Zheng and C. Xiao. 
% Free for education and research. 


Ntau=min(length(Tau),length(PdB));
if length(Tau)~=length(PdB)
    warning('Lengths of Tau and PdB are not equal. Truncate to the smaller one.')   
    Tau=Tau(1:Ntau);
    PdB=PdB(1:Ntau);
end

if length(Fd)< Ntau
    Fd(length(Fd)+1:Ntau)=Fd(length(Fd));
end
if Ns < max(Fd)*Tsym/Ts
    warning('Ns is too small to cover max Doppler spread.')
end

pwr= 10.^(PdB/10); % the total power will be normalized to 1.
Gain=sqrt(pwr/sum(pwr));% gain of each multipath ray 
FilterLength= 3; % Length/2 of pulse shaping filter: normally 3Tsym ~ 6Tsym
N_up=floor(Tsym/Ts); % upsampling factor
Ts=Tsym/N_up;

% total length of the channel taps is (-L1:L2)
L1=FilterLength*N_up;
L2=floor(max(Tau)/Ts)+L1;
Delay=floor(Tau/Tsym)*N_up;

Httau=zeros(L1+L2+1,Ns);
for l=1:Ntau
    t=[-L1:L1]*Ts-(Tau(l)-Delay(l)*Ts); % taking into account the fractional delay
    Ptau=RaisedCos(t,Tsym);% pulse shaping filter, row vector
    Gt=Gain(l)*RayleighFlat(Ns,Ts, Fd(l));% Doppler fading response, row vector
    Htemp=zeros(L1+L2+1,Ns);
    Htemp(Delay(l)+1:Delay(l)+2*L1+1,:)=Ptau'*Gt;
    Httau=Httau + Htemp;
end
GroupDelay=L1;